MODULE crsiom
   !!=====================================================================
   !!                    ***  MODULE  crs_iom ***
   !! Input/Output manager :  Library to read input files with NF90 (only fliocom module)
   !!====================================================================
   !! History :  9.0  ! 12 06  (J. Simeon) Original code
   !!                 ! 12 10  (C. Calone) Modified to finish code
   !!
   !!
   !!            kdompar_crs(1,1) = global x-dimension, jpiglo_crs
   !!            kdompar_crs(2,1) = global y-dimension, jpjglo_crs
   !!            kdompar_crs(:,2)  
   !!  from dom_oce INTEGER nimpp, njmpp      !: i- & j-indexes for mpp-subdomain left bottom
   !!
   !!  From LBC/mppini.F90      
   !!  ! No mpp computation
   !!   nimpp  = 1
   !!   njmpp  = 1
   !!   nlci   = jpi
   !!   nlcj   = jpj
   !!   nldi   = 1
   !!   nldj   = 1
   !!   nlei   = jpi
   !!   nlej   = jpj
   !!   nperio = jperio
   !!   nbondi = 2
   !!   nbondj = 2
   !!   nidom  = FLIO_DOM_NONE
   !!   npolj = jperio
   !!
   !! jes. 28 Jun 2012. TODO. make sure of variable declarations to be placed here or crs_dom.F90
   !!--------------------------------------------------------------------
   USE dom_oce         ! ocean space and time domain
   USE crs
   USE in_out_manager  ! I/O manager
   USE iom             ! I/O library


   IMPLICIT NONE
   PRIVATE

   PUBLIC crs_iom_open, crs_iom_rstput, crs_iom_put

   INTEGER                 ::   idomcrs     ! Type of domain to be written (default = jpdom_local_noovlap)
   LOGICAL                 ::   llnoov      ! local definition to read overlap


CONTAINS

   SUBROUTINE crs_iom_open( cdname, kiomid, ldwrt, kdom, kiolib )
      !!--------------------------------------------------------------------
      !!                       ***  MODULE crs_iom_open  ***
      !!
      !! ** Purpose : open an input file with NF90 on coarsened grid
      !!---------------------------------------------------------------------
      !! Arguments
      CHARACTER(len=*)       , INTENT(in)           ::   cdname   ! File name
      INTEGER                , INTENT(inout)        ::   kiomid   ! nf90 identifier of the opened file
      LOGICAL                , INTENT(in)           ::   ldwrt    ! read or write the file?
      INTEGER                , INTENT(in), OPTIONAL ::   kdom     ! Type of domain to be written (default = jpdom_local_noovlap)
      INTEGER                , INTENT(in), OPTIONAL ::   kiolib   ! library used to open the file (default = jpnf90)


      CALL dom_grid_crs   ! Save the parent grid information  & Switch to coarse grid domain

      llnoov = (jpni * jpnj ) == jpnij .AND. .NOT. lk_agrif 
      IF( llnoov ) THEN   ;   idomcrs = jpdom_local_noovlap   ! default definition
      ELSE                ;   idomcrs = jpdom_local_full      ! default definition
      ENDIF
      IF ( PRESENT(kdom) ) idomcrs = kdom

      CALL iom_open( cdname, kiomid, ldwrt, idomcrs, kiolib )

      CALL dom_grid_glo   ! Return to parent grid domain

   END SUBROUTINE crs_iom_open


   SUBROUTINE crs_iom_rstput( kt, kwrite, kiomid, cdvar, pv_r0d, pv_r1d, pv_r1d1, pv_r2d, pv_r3d, ktype )
      !!--------------------------------------------------------------------
      !!                   ***  SUBROUTINE  crs_iom_rp0d  ***
      !!
      !! ** Purpose : read the time axis cdvar in the file 
      !!--------------------------------------------------------------------
      ! Arguments
      INTEGER          ,                                  INTENT(in) :: kt       ! ocean time-step
      INTEGER          ,                                  INTENT(in) :: kwrite   ! writing time-step
      INTEGER          ,                                  INTENT(in) :: kiomid   ! Identifier of the file 
      CHARACTER(len=*) ,                                  INTENT(in) :: cdvar    ! variable name
      REAL(wp), OPTIONAL,                                 INTENT(in) :: pv_r0d   ! written Od field
      REAL(wp), OPTIONAL, DIMENSION(jpk),                 INTENT(in) :: pv_r1d   ! written 1d field
      REAL(wp), OPTIONAL, DIMENSION(jpiglo_crs),          INTENT(in) :: pv_r1d1   ! written 1d field !cc
      REAL(wp), OPTIONAL, DIMENSION(jpi_crs,jpj_crs) ,    INTENT(in) :: pv_r2d   ! written 2d field
      REAL(wp), OPTIONAL, DIMENSION(jpi_crs,jpj_crs,jpk), INTENT(in) :: pv_r3d   ! written 3d field
      INTEGER,  OPTIONAL,                                 INTENT(in) :: ktype    ! variable type (default R8)

      ! Local
      INTEGER               :: ivid     ! variable id
      INTEGER               :: itype    ! variable type

      CALL dom_grid_crs   ! Save the parent grid information  & Switch to coarse grid domain

      IF(     PRESENT(pv_r0d) ) THEN ; CALL iom_rstput( kt, kwrite, kiomid, cdvar, pv_r0d, ktype )
      ELSEIF( PRESENT(pv_r1d) ) THEN ; CALL iom_rstput( kt, kwrite, kiomid, cdvar, pv_r1d, ktype )
      ELSEIF( PRESENT(pv_r1d1)) THEN ; CALL iom_rstput( kt, kwrite, kiomid, cdvar, pv_r1d1, ktype ) !cc
      ELSEIF( PRESENT(pv_r2d) ) THEN ; CALL iom_rstput( kt, kwrite, kiomid, cdvar, pv_r2d, ktype )
      ELSEIF( PRESENT(pv_r3d) ) THEN ; CALL iom_rstput( kt, kwrite, kiomid, cdvar, pv_r3d, ktype )
      ENDIF

      CALL dom_grid_glo   ! Return to parent grid domain
 
   END SUBROUTINE crs_iom_rstput

   SUBROUTINE crs_iom_put( cdvar , pv_r0d, pv_r1d, pv_r2d, pv_r3d )
      !!--------------------------------------------------------------------
      !!                   ***  SUBROUTINE  iom_put  ***
      !!
      !! ** Purpose : read the time axis cdvar in the file 
      !!--------------------------------------------------------------------
      ! Arguments
      CHARACTER(len=*)            , INTENT(in)           ::   cdvar    ! variable name
      REAL(wp)                    , INTENT(in), OPTIONAL ::   pv_r0d   ! written Od field
      REAL(wp), DIMENSION(      :), INTENT(in), OPTIONAL ::   pv_r1d   ! written 1d field
      REAL(wp), DIMENSION(:, :   ), INTENT(in), OPTIONAL ::   pv_r2d   ! written 2d field
      REAL(wp), DIMENSION(:, :, :), INTENT(in), OPTIONAL ::   pv_r3d   ! written 3d field

      ! Local
      INTEGER               :: ivid     ! variable id
      INTEGER               :: itype    ! variable type

      CALL dom_grid_crs   ! Save the parent grid information  & Switch to coarse grid domain

      ! variable definition
      IF(     PRESENT(pv_r0d) ) THEN   ;  CALL iom_put( cdvar, pv_r0d )
      ELSEIF( PRESENT(pv_r1d) ) THEN   ;  CALL iom_put( cdvar, pv_r1d )
      ELSEIF( PRESENT(pv_r2d) ) THEN   ;  CALL iom_put( cdvar, pv_r2d )
      ELSEIF( PRESENT(pv_r3d) ) THEN   ;  CALL iom_put( cdvar, pv_r3d )
      ENDIF

      CALL dom_grid_glo   ! Return to parent grid domain

   END SUBROUTINE crs_iom_put

END MODULE crsiom
